package de.fhmuenster.masterthesis.generator;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import de.fhmuenster.masterthesis.testgeneratorDSL.FlowElement;

public class BPMNGenFlow {

	private String name;
	private List<BPMNGenFlowElement> flowElements;
	private List<BPMNGenFlowElement> exclusiveFlowElements;
	private BPMNGenDiagram bpmnDiagram;
	
	public BPMNGenFlow() {
		flowElements = new ArrayList<>();
	}
	
	public BPMNGenFlow(List<BPMNGenFlowElement> flowElements) {
		this.flowElements = flowElements;
	}
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public List<BPMNGenFlowElement> getFlowElements() {
		return flowElements;
	}
	
	public void setFlowElements(List<BPMNGenFlowElement> flowElements) {
		this.flowElements = flowElements;
	}
	
	public void appendFlowElement(BPMNGenFlowElement flowElement) {
		flowElements.add(flowElement);
	}
	
	public BPMNGenFlowElement getFirstNode() {
		return (BPMNGenFlowElement) flowElements.get(0);
	}
	
	public BPMNGenFlowElement getLastNode() {
		int index = flowElements.size() - 1;
		return flowElements.get(index);
	}
	
	public List<BPMNGenFlowElement> getExclusiveFlowElements() {
		return exclusiveFlowElements;
	}
	
	public List<BPMNGenFlowElement> getExclusiveFlowNodes() {
		return exclusiveFlowElements.stream() //
				.filter(flowElement -> isFlowNode(flowElement)) //
				.collect(Collectors.toList()); //
	}
	
	public void setExclusiveFlowElements(List<BPMNGenFlowElement> exclusiveFlowElements) {
		this.exclusiveFlowElements = exclusiveFlowElements;
	}
	
	public BPMNGenDiagram getBpmnDiagram() {
		return bpmnDiagram;
	}
	
	public void setBpmnDiagram(BPMNGenDiagram bpmnDiagram) {
		this.bpmnDiagram = bpmnDiagram;
	}
	
	public List<BPMNGenFlowElement> getFlowNodes() {
		return flowElements.stream() //
				.filter(flowElement -> isFlowNode(flowElement)) //
				.collect(Collectors.toList()); //
	}
	
	private boolean isFlowNode(BPMNGenFlowElement flowElement) {
		return !flowElement.getType().equals(FlowElementType.SEQUENCE_FLOW);
	}
}
