package de.fhmuenster.masterthesis.serialization;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import de.fhmuenster.masterthesis.testgeneratorDSL.BPMNDiagram;
import de.fhmuenster.masterthesis.testgeneratorDSL.BPMNReference;
import de.fhmuenster.masterthesis.testgeneratorDSL.BooleanCompare;
import de.fhmuenster.masterthesis.testgeneratorDSL.BooleanCompareSymbol;
import de.fhmuenster.masterthesis.testgeneratorDSL.BooleanVariable;
import de.fhmuenster.masterthesis.testgeneratorDSL.BooleanVariableDeclaration;
import de.fhmuenster.masterthesis.testgeneratorDSL.BooleanVariableEquals;
import de.fhmuenster.masterthesis.testgeneratorDSL.BooleanVariableNotEquals;
import de.fhmuenster.masterthesis.testgeneratorDSL.BusinessRuleTaskFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.Compare;
import de.fhmuenster.masterthesis.testgeneratorDSL.DelegateMock;
import de.fhmuenster.masterthesis.testgeneratorDSL.DelegateServiceTaskFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.DelegateServiceTaskReference;
import de.fhmuenster.masterthesis.testgeneratorDSL.EndCheck;
import de.fhmuenster.masterthesis.testgeneratorDSL.EndChecks;
import de.fhmuenster.masterthesis.testgeneratorDSL.EndFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.ExternalServiceMock;
import de.fhmuenster.masterthesis.testgeneratorDSL.ExternalServiceTaskFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.ExternalTopic;
import de.fhmuenster.masterthesis.testgeneratorDSL.ExternalTopicReference;
import de.fhmuenster.masterthesis.testgeneratorDSL.Flow;
import de.fhmuenster.masterthesis.testgeneratorDSL.FlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.FlowElementReference;
import de.fhmuenster.masterthesis.testgeneratorDSL.FlowReference;
import de.fhmuenster.masterthesis.testgeneratorDSL.GatewayFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.IntCompare;
import de.fhmuenster.masterthesis.testgeneratorDSL.IntCompareSymbol;
import de.fhmuenster.masterthesis.testgeneratorDSL.IntVariable;
import de.fhmuenster.masterthesis.testgeneratorDSL.IntVariableDeclaration;
import de.fhmuenster.masterthesis.testgeneratorDSL.IntVariableEquals;
import de.fhmuenster.masterthesis.testgeneratorDSL.IntVariableNotEquals;
import de.fhmuenster.masterthesis.testgeneratorDSL.Loop;
import de.fhmuenster.masterthesis.testgeneratorDSL.ManualMock;
import de.fhmuenster.masterthesis.testgeneratorDSL.ManualTaskFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.Mock;
import de.fhmuenster.masterthesis.testgeneratorDSL.MockReference;
import de.fhmuenster.masterthesis.testgeneratorDSL.MockingRule;
import de.fhmuenster.masterthesis.testgeneratorDSL.Model;
import de.fhmuenster.masterthesis.testgeneratorDSL.ScriptTaskFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.SequenceFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.ServiceTaskFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.StartFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.StringCompare;
import de.fhmuenster.masterthesis.testgeneratorDSL.StringCompareSymbol;
import de.fhmuenster.masterthesis.testgeneratorDSL.StringVariable;
import de.fhmuenster.masterthesis.testgeneratorDSL.StringVariableDeclaration;
import de.fhmuenster.masterthesis.testgeneratorDSL.StringVariableEquals;
import de.fhmuenster.masterthesis.testgeneratorDSL.StringVariableNotEquals;
import de.fhmuenster.masterthesis.testgeneratorDSL.Test;
import de.fhmuenster.masterthesis.testgeneratorDSL.TestgeneratorDSLFactory;
import de.fhmuenster.masterthesis.testgeneratorDSL.UserTaskFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.Variable;
import de.fhmuenster.masterthesis.testgeneratorDSL.VariableDeclaration;
import de.fhmuenster.masterthesis.testgeneratorDSL.VariableDeclarations;
import de.fhmuenster.masterthesis.testgeneratorDSL.VariableEquals;
import de.fhmuenster.masterthesis.testgeneratorDSL.VariableNotEquals;
import de.fhmuenster.masterthesis.testgeneratorDSL.VariableReference;
import de.fhmuenster.masterthesis.utils.TestgeneratorDSLUtils;

public class TestgeneratorDSLObjectCreator {

	private TestgeneratorDSLObjectCreator() {
	}
	
	public static Model createModel() {
		Model model = TestgeneratorDSLFactory.eINSTANCE.createModel();
		
		return model;
	}
	
	public static BPMNDiagram createBPMNDiagram(String name, String testpackage, String bpmn, String key, List<String> dmns, List<String> embeddeds) {
		BPMNDiagram bpmnDiagram = TestgeneratorDSLFactory.eINSTANCE.createBPMNDiagram();
		bpmnDiagram.setName(name);
		bpmnDiagram.setTestpackage(testpackage);
		bpmnDiagram.setBpmn(bpmn);
		bpmnDiagram.setKey(key);
		bpmnDiagram.getDmn().addAll(dmns);
		bpmnDiagram.getEmbedded().addAll(embeddeds);
		
		return bpmnDiagram;
	}
	
	public static StartFlowElement createStartFlowElement(String name, List<Variable> startVariables) {
		StartFlowElement element = TestgeneratorDSLFactory.eINSTANCE.createStartFlowElement();
		element.setName(name);
		
		for (Variable startVar : startVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(startVar);
			element.getInputVariables().add(variableReference);
		}

		return element;
	}
	
	public static EndFlowElement createEndFlowElement(String name) {
		EndFlowElement element = TestgeneratorDSLFactory.eINSTANCE.createEndFlowElement();
		element.setName(name);

		return element;
	}
	
	public static UserTaskFlowElement createUserTaskFlowElement(String name, List<Variable> inputVariables, List<Variable> outputVariables, String label) {
		UserTaskFlowElement element = TestgeneratorDSLFactory.eINSTANCE.createUserTaskFlowElement();
		element.setName(name);
		element.setLabel(label);

		for (Variable inputVariable : inputVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(inputVariable);
			element.getInputVariables().add(variableReference);
		}
		
		for (Variable outputVariable : outputVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(outputVariable);
			element.getOutputVariables().add(variableReference);
		}
		
		return element;
	}
	
	public static ManualTaskFlowElement createManualTaskFlowElement(String name, List<Variable> inputVariables, List<Variable> outputVariables, String label) {
		ManualTaskFlowElement element = TestgeneratorDSLFactory.eINSTANCE.createManualTaskFlowElement();
		element.setName(name);
		element.setLabel(label);

		for (Variable inputVariable : inputVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(inputVariable);
			element.getInputVariables().add(variableReference);
		}
		
		for (Variable outputVariable : outputVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(outputVariable);
			element.getOutputVariables().add(variableReference);
		}
		
		return element;
	}
	
	public static ScriptTaskFlowElement createScriptTaskFlowElement(String name, List<Variable> inputVariables, List<Variable> outputVariables, String label) {
		ScriptTaskFlowElement element = TestgeneratorDSLFactory.eINSTANCE.createScriptTaskFlowElement();
		element.setName(name);
		element.setLabel(label);
		
		for (Variable inputVariable : inputVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(inputVariable);
			element.getInputVariables().add(variableReference);
		}
		
		for (Variable outputVariable : outputVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(outputVariable);
			element.getOutputVariables().add(variableReference);
		}

		return element;
	}
	
	public static ServiceTaskFlowElement createServiceTaskFlowElement(String name) {
		ServiceTaskFlowElement element = TestgeneratorDSLFactory.eINSTANCE.createServiceTaskFlowElement();
		element.setName(name);
		
		return element;
	}
	
	public static ServiceTaskFlowElement createDelegateServiceTaskFlowElement(String name, String delegate, List<Variable> inputVariables, List<Variable> outputVariables, String label) {
		DelegateServiceTaskFlowElement element = TestgeneratorDSLFactory.eINSTANCE.createDelegateServiceTaskFlowElement();
		element.setName(name);
		element.setLabel(label);
		element.setDelegate(delegate);
		
		for (Variable inputVariable : inputVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(inputVariable);
			element.getInputVariables().add(variableReference);
		}
		
		for (Variable outputVariable : outputVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(outputVariable);
			element.getOutputVariables().add(variableReference);
		}

		return element;
	}
	
	public static ServiceTaskFlowElement createExternalServiceTaskFlowElement(String name, ExternalTopic externalTopic, List<VariableDeclaration> hardcodedInputs, 
			String label, List<Variable> inputVariables, List<Variable> outputVariables) {
		ExternalTopicReference externalTopicRef = TestgeneratorDSLFactory.eINSTANCE.createExternalTopicReference();
		externalTopicRef.setRef(externalTopic);
		
		ExternalServiceTaskFlowElement element = TestgeneratorDSLFactory.eINSTANCE.createExternalServiceTaskFlowElement();
		element.setName(name);
		element.setLabel(label);
		element.setExternalTopic(externalTopicRef);
		element.getHardcodedInputs().addAll(hardcodedInputs);
		for (Variable inputVariable : inputVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(inputVariable);
			element.getInputVariables().add(variableReference);
		}
		
		for (Variable outputVariable : outputVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(outputVariable);
			element.getOutputVariables().add(variableReference);
		}

		return element;
	}
	
	public static BusinessRuleTaskFlowElement createBusinessRuleTaskFlowElement(String name, String resultVariable, List<Variable> inputVariables, List<Variable> outputVariables, String label) {
		BusinessRuleTaskFlowElement element = TestgeneratorDSLFactory.eINSTANCE.createBusinessRuleTaskFlowElement();
		element.setName(name);
		element.setResultVariable(resultVariable);
		element.setLabel(label);
		
		for (Variable inputVariable : inputVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(inputVariable);
			element.getInputVariables().add(variableReference);
		}
		
		for (Variable outputVariable : outputVariables) {
			VariableReference variableReference = TestgeneratorDSLFactory.eINSTANCE.createVariableReference();
			variableReference.setRef(outputVariable);
			element.getOutputVariables().add(variableReference);
		}
		
		return element;
	}
	
	public static GatewayFlowElement createGatewayFlowElement(String name) {
		GatewayFlowElement element = TestgeneratorDSLFactory.eINSTANCE.createGatewayFlowElement();
		element.setName(name);

		return element;
	}
	
	public static SequenceFlowElement createSequenceFlowElement(String name, String expression) {
		SequenceFlowElement element = TestgeneratorDSLFactory.eINSTANCE.createSequenceFlowElement();
		element.setName(name);
		element.setExpression(expression);

		return element;
	}
	
	public static Flow createFlow(String name, BPMNDiagram bpmn, List<FlowElement> inclElements, List<FlowElement> exclElements) {
		BPMNReference bpmnReference = TestgeneratorDSLFactory.eINSTANCE.createBPMNReference();
		bpmnReference.setRef(bpmn);
		
		Flow flow = TestgeneratorDSLFactory.eINSTANCE.createFlow();
		flow.setName(name);
		flow.setBpmnReference(bpmnReference);
		
		List<FlowElementReference> inclElementRefs = new ArrayList<>();
		List<FlowElementReference> exclElementRefs = new ArrayList<>();
		for (FlowElement inclElement : inclElements) {
			FlowElementReference ref = TestgeneratorDSLFactory.eINSTANCE.createFlowElementReference();
			ref.setRef(inclElement);
			inclElementRefs.add(ref);
		}
		
		for (FlowElement exclElement : exclElements) {
			FlowElementReference ref = TestgeneratorDSLFactory.eINSTANCE.createFlowElementReference();
			ref.setRef(exclElement);
			exclElementRefs.add(ref);
		}
		
		flow.getInclElements().addAll(inclElementRefs);
		flow.getExclElements().addAll(exclElementRefs);

		return flow;
	}

	public static StringVariable createStringVariable(String name, List<String> proposals) {
		StringVariable stringVariable = TestgeneratorDSLFactory.eINSTANCE.createStringVariable();
		stringVariable.setName(name);
		stringVariable.getProposals().addAll(proposals);

		return stringVariable;
	}
	
	public static IntVariable createIntVariable(String name, List<Integer> proposals) {
		IntVariable intVariable = TestgeneratorDSLFactory.eINSTANCE.createIntVariable();
		intVariable.setName(name);
		intVariable.getProposals().addAll(proposals);

		return intVariable;
	}
	
	public static BooleanVariable createBooleanVariable(String name, List<Boolean> proposals) {
		BooleanVariable booleanVariable = TestgeneratorDSLFactory.eINSTANCE.createBooleanVariable();
		booleanVariable.setName(name);
		booleanVariable.getProposals().addAll(TestgeneratorDSLUtils.mapBoolean(proposals));

		return booleanVariable;
	}
	
	public static ExternalServiceMock createExternalServiceMock(String name, ExternalTopic externalTopic, List<MockingRule> mockRules) {
		ExternalTopicReference externalTopicRef = TestgeneratorDSLFactory.eINSTANCE.createExternalTopicReference();
		externalTopicRef.setRef(externalTopic);
		
		ExternalServiceMock mock = TestgeneratorDSLFactory.eINSTANCE.createExternalServiceMock();
		mock.setName(name);
		mock.setExternalTopic(externalTopicRef);
		mock.getMockingRules().addAll(mockRules);
		
		return mock;
	}
	
	public static MockingRule createMockingRule(List<Compare> comparisons, List<VariableDeclaration> variables) {
		MockingRule mockingRule = TestgeneratorDSLFactory.eINSTANCE.createMockingRule();
		mockingRule.getComparisons().addAll(comparisons);
		mockingRule.getVariables().addAll(variables);
		
		return mockingRule;
	}
	
	public static DelegateMock createDelegateMock(String name, DelegateServiceTaskFlowElement service, String delegateClass, List<ManualMock> manualMocks) {
		DelegateServiceTaskReference serviceRef = TestgeneratorDSLFactory.eINSTANCE.createDelegateServiceTaskReference();
		serviceRef.setRef(service);

		DelegateMock mock = TestgeneratorDSLFactory.eINSTANCE.createDelegateMock();
		mock.setName(name);
		mock.setDelegateClass(delegateClass);
		mock.setServiceTaskReference(serviceRef);
		mock.getManualMocks().addAll(manualMocks);
		
		return mock;
	}
	
	public static ManualMock createManualMock(String serviceName, String setter) {
		ManualMock manualMock = TestgeneratorDSLFactory.eINSTANCE.createManualMock();
		manualMock.setServiceName(serviceName);
		manualMock.setSetter(setter);
		
		return manualMock;
	}
	
	public static Test createTest(String name, Flow flow, List<VariableDeclarations> variableDeclarations, EndChecks endChecks, List<Mock> mocks, int priority) {
		FlowReference flowReference = TestgeneratorDSLFactory.eINSTANCE.createFlowReference();
		flowReference.setRef(flow);
		
		List<MockReference> mockRefs = new ArrayList<>();
		mocks.stream().forEach(mock -> {
			MockReference mockReference = TestgeneratorDSLFactory.eINSTANCE.createMockReference();
			mockReference.setRef(mock);
			
			mockRefs.add(mockReference);
		});
		
		Test test = TestgeneratorDSLFactory.eINSTANCE.createTest();
		test.setName(name);
		test.setFlowReference(flowReference);
		test.setPriority(priority);
		test.getDeclarations().addAll(variableDeclarations);
		test.getMocks().addAll(mockRefs);

		if(endChecks != null && !endChecks.getEndChecks().isEmpty()) {
			test.setEndCheck(endChecks);
		}
		
		return test;
	}
	
	public static Loop createLoop(String id, String startId, String followerId, String endId, int complexity, List<Integer> frequencies) {
		Loop loop = TestgeneratorDSLFactory.eINSTANCE.createLoop();
		loop.setName(id);
		loop.setStart(startId);
		loop.setFollower(followerId);
		loop.setEnd(endId);
		loop.setComplexity(complexity);
		loop.getFrequencies().addAll(frequencies);
		
		return loop;
	}
	
	public static VariableDeclarations createVariableDeclarations(FlowElement flowElement, List<VariableDeclaration> variables) {
		VariableDeclarations variableDeclarations = TestgeneratorDSLFactory.eINSTANCE.createVariableDeclarations();
		
		FlowElementReference reference = TestgeneratorDSLFactory.eINSTANCE.createFlowElementReference();
		reference.setRef(flowElement);
		variableDeclarations.setTaskReference(reference);
		variableDeclarations.getVariables().addAll(variables);
		
		return variableDeclarations;
	}
	
	public static VariableDeclaration createVariableDeclaration(StringVariable variable, String value) {
		StringVariableDeclaration variableDeclaration = TestgeneratorDSLFactory.eINSTANCE.createStringVariableDeclaration();
		variableDeclaration.setKey(variable);
		variableDeclaration.setValue(value);
		
		return variableDeclaration;
	}
	
	public static VariableDeclaration createVariableDeclaration(IntVariable variable, Integer value) {
		IntVariableDeclaration variableDeclaration = TestgeneratorDSLFactory.eINSTANCE.createIntVariableDeclaration();
		variableDeclaration.setKey(variable);
		variableDeclaration.setValue(value);
		
		return variableDeclaration;
	}
	
	public static VariableDeclaration createVariableDeclaration(BooleanVariable variable, Boolean value) {
		BooleanVariableDeclaration variableDeclaration = TestgeneratorDSLFactory.eINSTANCE.createBooleanVariableDeclaration();
		variableDeclaration.setKey(variable);
		variableDeclaration.setValue(TestgeneratorDSLUtils.mapBoolean(value));
		
		return variableDeclaration;
	}
	
	public static Compare createCompare(StringVariable variable, String value, StringCompareSymbol operator) {
		StringCompare compare = TestgeneratorDSLFactory.eINSTANCE.createStringCompare();
		compare.setKey(variable);
		compare.setValue(value);
		compare.setCompareSymbol(operator);
		
		return compare;
	}
	
	public static Compare createCompare(IntVariable variable, Integer value, IntCompareSymbol operator) {
		IntCompare compare = TestgeneratorDSLFactory.eINSTANCE.createIntCompare();
		compare.setKey(variable);
		compare.setValue(value);
		compare.setCompareSymbol(operator);
		
		return compare;
	}
	
	public static Compare createCompare(BooleanVariable variable, Boolean value, BooleanCompareSymbol operator) {
		BooleanCompare compare = TestgeneratorDSLFactory.eINSTANCE.createBooleanCompare();
		compare.setKey(variable);
		compare.setValue(TestgeneratorDSLUtils.mapBoolean(value));
		compare.setCompareSymbol(operator);
		
		return compare;
	}
	
	public static EndChecks createEndChecks(List<EndCheck> endCheckList) {
		EndChecks endChecks = TestgeneratorDSLFactory.eINSTANCE.createEndChecks();
		endChecks.getEndChecks().addAll(endCheckList);
		
		return endChecks;
	}
	
	public static VariableEquals createVariableEquals(StringVariable variable, String value) {
		StringVariableEquals checkEqual = TestgeneratorDSLFactory.eINSTANCE.createStringVariableEquals();
		checkEqual.setKey(variable);
		checkEqual.setValue(value);
		
		return checkEqual;
	}
	
	public static VariableEquals createVariableEquals(IntVariable variable, Integer value) {
		IntVariableEquals checkEqual = TestgeneratorDSLFactory.eINSTANCE.createIntVariableEquals();
		checkEqual.setKey(variable);
		checkEqual.setValue(value);
		
		return checkEqual;
	}
	
	public static VariableEquals createVariableEquals(BooleanVariable variable, Boolean value) {
		BooleanVariableEquals checkEqual = TestgeneratorDSLFactory.eINSTANCE.createBooleanVariableEquals();
		checkEqual.setKey(variable);
		checkEqual.setValue(TestgeneratorDSLUtils.mapBoolean(value));
		
		return checkEqual;
	}
	
	public static VariableNotEquals createVariableNotEquals(StringVariable variable, String value) {
		StringVariableNotEquals checkNotEqual = TestgeneratorDSLFactory.eINSTANCE.createStringVariableNotEquals();
		checkNotEqual.setKey(variable);
		checkNotEqual.setValue(value);
		
		return checkNotEqual;
	}
	
	public static VariableNotEquals createVariableNotEquals(IntVariable variable, Integer value) {
		IntVariableNotEquals checkNotEqual = TestgeneratorDSLFactory.eINSTANCE.createIntVariableNotEquals();
		checkNotEqual.setKey(variable);
		checkNotEqual.setValue(value);
		
		return checkNotEqual;
	}
	
	public static VariableNotEquals createVariableNotEquals(BooleanVariable variable, Boolean value) {
		BooleanVariableNotEquals checkNotEqual = TestgeneratorDSLFactory.eINSTANCE.createBooleanVariableNotEquals();
		checkNotEqual.setKey(variable);
		checkNotEqual.setValue(TestgeneratorDSLUtils.mapBoolean(value));
		
		return checkNotEqual;
	}

	public static ExternalTopic createExternalTopic(String externalTopicTextual) {
		ExternalTopic externalTopic = TestgeneratorDSLFactory.eINSTANCE.createExternalTopic();
		externalTopic.setName(externalTopicTextual);
		
		return externalTopic;
	}
	
	
}
